function descriptors = createDescriptors(baseName, options, regList)
% author: dino bellugi
% copyright: blueG soft wear 2010
% processes image, gPb and UCM, to produce all descriptors
% inputs:
%   baseName = base name for files
%   options = logical(1,6) for print, plot, save (general & individual)
%   regList = row vector with indices of regions (default: all regions)
% outputs:
%   descriptors = descriptors matrix [size(descriptor,1), size(regList,2)]
% examples:
%   descriptors = createDescriptors('../newData/Img0057_cropped', ...
%       [true(1,3) false(1,3)], region_list); % just listed regions
%   descriptors = createDescriptors('../newData/Img0057_cropped', ...
%       [true(1,3) false(1,3)]); % all regions
% notes:
%   has a bunch of hard-coded parameters (see below) in particular:
%       selection = logical(22), selects which properties to include
% naming convention:
%   imageName = [baseName '.bmp'];
%   regionsName = [baseName '_regions'];
%   gPbName = [baseName '_gPb.mat'];
%   descriptorsName = [baseName '_descriptors']

    % check command line
    if (nargin < 1 || nargin > 3)
        error('Usage: descriptors = createDescriptors(baseName, options, [regList])\n');
    end
    if (nargin < 3)
        regList = '';
    end
    if (nargin < 2)
        options = true(1,6);
    end

    %% add paths
    addLibs();
    
    %% setup naming structure
    imageName = [baseName '.bmp'];
    regionsName = [baseName '_regions'];
    gPbName = [baseName '_gPb.mat'];
    descriptorsName = [baseName '_descriptors'];

    %% options:
    % options(1) = analyze all regions

    %% selection of properties in descriptor
    % selection(1): Area
    % selection(2): Perimeter
    % selection(3): Perimeter/Area
    % selection(4): Eccentricity
    % selection(5): Solidity
    % selection(6): Circularity
    % selection(7): Orientation histograms
    % selection(8): Change of orientation histograms
    % selection(9): Boundary quality histograms
    % selection(10): Color histograms (Red, Green, Blue)
    % selection(11): Super-pixel color histograms (Red, Green, Blue)
    % selection(12): Intensity histogram (Gray)
    % selection(13): Super-pixel intensity histogram (Gray)
    % selection(14): Texton map histogram (Gray)
    % selection(15): Range filter histogram (Gray)
    % selection(16): Super-pixel range filter histogram (Gray)
    % selection(17): Entropy filter histogram (Gray)
    % selection(18): Super-pixel entropy filter histogram (Gray)
    % selection(19): Gray-Level Co-Occurrence Matrix Correlation
    % selection(20): Super-pixel Gray-Level Co-Occurrence Matrix Correlation
    % selection(21): Region entropy
    % selection(22): Super-pixel region entropy
    % choose here:      [1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2]
    selection = logical([1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1]);

    %% read and display data
    load(regionsName);
    load(gPbName);
    clear gPb_orient
    labels = zeros(size(ucm));
    for r = 1:numel(regions)
        if scale(r)==thresh,
            labels(regions{r})=r;
        end
    end
    
    % get region list
    if (strcmpi(regList, ''))
        if (options(1))
            fprintf('Analyzing all regions ...\n');
        end
        regList = [1:numel(regions)];
    end
    
    % colormap
    mp=rand(256,3);
    
    % get image
    rgb_img = imread(imageName);
    gray_img = rgb2gray(rgb_img);
    
    if (options(2))
        
        figure;imshow(rgb_img);title('Original image');
        figure;imshow(gPb_thin);title('gPb map');
        figure;imshow(ucm);title('UCM map');
        figure;imshow(labels,[]);colormap(mp);title(['Regions at level ', num2str(thresh)]);
        impixelinfo
        figure;imshow(imageName);
        hold on
        alpha_mask=zeros(size(labels))+0.3;
        h = imshow(labels,[]);colormap(mp);
        set(h,'AlphaData',alpha_mask);
        hold off
        impixelinfo
    end

    %% pick a region collection and compute some stuff
    num_reg = size(regList,2);
    hWB = waitbar(0, 'Construting descriptors ...');
    warning off;
    
    for ii = 1:num_reg
        tic
        
        reg = regList(ii);

        if (options(1))
            fprintf(['Selecting region ', num2str(reg), ' ...\n']);
        end

        % get mask
        reg_mask = regions{reg};
        [rows cols] = size(reg_mask);
        mask(:,:,1) = reg_mask;
        mask(:,:,2) = reg_mask;
        mask(:,:,3) = reg_mask;

        % rgb region
        rgb_reg = immultiply(rgb_img, mask);

        % gray region
        gray_reg = rgb2gray(rgb_reg);

        % create copies with NaN instead of zero
%         rgb_reg_nan = double(rgb_reg);
        r_reg_nan = double(rgb_reg(:,:,1));
        g_reg_nan = double(rgb_reg(:,:,2));
        b_reg_nan = double(rgb_reg(:,:,3));
        gray_reg_nan = double(gray_reg);
        text_reg_nan = double(textons);
%         rgb_reg_nan2 = double(rgb_reg);
%         gray_reg_nan2 = double(gray_reg);
%         text_reg_nan2 = double(textons);
%         for i=1:rows
%             for j=1:cols
%                 if (~reg_mask(i,j))
%                     rgb_reg_nan(i,j,:) = NaN;
%                     gray_reg_nan(i,j) = NaN;
%                     text_reg_nan(i,j) = NaN;
%                 end
%             end
%         end
        
        r_reg_nan(~reg_mask) = NaN;
        g_reg_nan(~reg_mask) = NaN;
        b_reg_nan(~reg_mask) = NaN;
        rgb_reg_nan = cat(3, r_reg_nan, g_reg_nan, b_reg_nan);
        gray_reg_nan(~reg_mask) = NaN;
        text_reg_nan(~reg_mask) = NaN;

        % plot
        if (options(2))
            figure;imshow(rgb_reg);
            title(['Regions collection ', num2str(reg), ' (RGB)']);
            figure;imshow(gray_reg);
            title(['Regions collection ', num2str(reg), ' (grayscale)']);
        end
        
        % make descriptor
        [desc descriptorSize] = mkDescriptor(rgb_img, rgb_reg, rgb_reg_nan, ...
            gray_img, gray_reg, gray_reg_nan, gPb_thin, text_reg_nan, ...
            reg_mask, reg, selection, baseName, options(4:6));

        if (ii == 1)
            descriptors = zeros(descriptorSize, num_reg, 'double');
        end
        
        % add descriptor to list
        descriptors(:,ii) = desc';

        if (options(2))
            close all
        end
        
        toc
        waitbar(ii/num_reg, hWB);
    end

    close(hWB);
    
    if (options(3))
        save(descriptorsName, 'descriptors');
    end
    warning on
    
end