function doClassify(baseName, type, method, order, mode)
%% main classification function
% author: dino bellugi
% copyright: blueG soft wear 2010
% takes in the data structure with region tree
% creates a desriptor for every node
% calls the SVM classification using trained model
% assigns labels and scores to the nodes
% loops through all leaf-root paths picking best score
% modifies the tree as selections are made
% saves resulting annotated and measured data
%
% baseName is a string specifying the file naming structure used for the
% image.
%
% type is an optional int indicating the svm kernel type:
% 0: linear, 1: poly, 2: rbf (default), 3: sigmoid.
%
% method is an optional int indicating the tree pruning scheme:
% 0: standard, 1: greedy paths, 2: greedy voting (default), more to be added.
%
% order is an optional int indicating what to choose for non-unique values:
%   0: the first (smallest stone)
%   1: the last (largest stone)
%   2: the most solid and circular (using the descriptor), default.
% neither method works well, must find another solution!
%
% mode is an optional logical indicating if the descriptors should be
% generated (true) or is they alread exist (false, default).
%
% example:
%   doClassify('..\newData\Img0158_cropped')
%

%% init
    % check command line
    if (nargin < 1 || nargin > 5)
        error('Usage: doClassify(baseName, [type, method, order, mode])');
    end
    
    if (nargin < 5)
        mode = false;
    end
    
    if (nargin < 4)
        order = 2;
    end
    
    if (nargin < 3)
        method = 2;
    end
    
    if (nargin < 2)
        type = 2;
    end
        
    % file names
    initName = [baseName '_initS.mat'];
    imageName = [baseName '.bmp'];
    finalName = [baseName '_finalS.mat'];
    statsName = [baseName '_regionProps.mat'];
    descriptorsName = [baseName '_descriptors'];
    
    % add paths
    addLibs();
    
    % initialize data structure
    fprintf(['Initializing structures for ', baseName, ' ...\n']);
    S = initS(baseName);

    % save data structure
    save(initName, 'S');
    
%% descriptors
    % make descriptors (these might be already made, so check)
    if (mode)
        fprintf(['creating descriptors for ', baseName, ' ...\n']);
        % options is 1x6 print plot save (high level) print plot save (low level)
        % save option must be on as the exchange is via file
        tic
        createDescriptors(baseName, [1 0 1 0 0 0]);
        toc
    end

%% classify   
    % classify and annotate data structure
    fprintf(['Classifying ', baseName, ' ...\n']);
    
    % options is 1x2 print save
    [S.labels, S.accuracy, S.value] = classifySVM(baseName, type, [1 1]);
    
    % change 0's in labels to -1 for gui to visualize them
    % note: comment this when doTrain is changed to pass labels as -1,1
    % S.labels(S.labels == 0) = -1;
    
    % change values from 2-column class probability to "confidence"
    S.value = max(S.value,[],2);
    
%     The function 'svmpredict' has three outputs. The first one,
%     predictd_label, is a vector of predicted labels. The second output,
%     accuracy, is a vector including accuracy (for classification), mean
%     squared error, and squared correlation coefficient (for regression).
%     The third is a matrix containing decision values or probability
%     estimates (if '-b 1' is specified). If k is the number of classes,
%     for decision values, each row includes results of predicting
%     k(k-1/2) binary-class SVMs. For probabilities, each row contains k values
%     indicating the probability that the testing instance is in each class.
%     Note that the order of classes here is the same as 'Label' field
%     in the model structure.
    
%% prune the tree (select the stones in each path and re-arrange the tree)
% different greedy methods will be implemented here.
%
% the variable method will indicate which to use.
%
% the first method (method = 0), goes through the paths in standard order.
%
% the second method (method = 1), sorts the paths by average level
% distance (could be substituted with min or max distance in initS).
%
% the third method has every path vote for their best stone, then the
% greedy selection starts from the stone receiving most votes.

    % load descriptors if necessary
    if (order == 2)
        fprintf(['Loading descriptors for ', baseName, ' ...\n']);
        load(descriptorsName);
    else
        myDescr = [];
    end

    % do the selection/pruning
    fprintf(['Pruning region tree for ', baseName, ' ...\n']);
    tic

	n = S.numPaths;
    
    switch (method)
    
        % unsorted paths
        case 0

            % loop over all paths
            for i=1:n

                % extract path
                myPath = extractPath(S, i);
                
                % extract descriptors (if necessary)
                if (order == 2)
                    myDescr = descriptors(:, myPath);
                end
                
                % find best score
                bestInd = bestInPath(S, myPath, order, myDescr);

                if (~bestInd)
                    continue;
                end

                % label it a stone
                S.labels(bestInd) = 1;

                % set children to -1
                S = setChildrenValue(S,bestInd,-1);

                % unhook children of parents and set parents to -1
                S = unhookSibs(S, bestInd, S.root, true);
            end

        % sorted paths
        case 1

            % loop over sorted paths
            for i=1:n

                % extract path
                myPath = extractPath(S, S.paths.sortedPaths(i));
                
                % extract descriptors (if necessary)
                if (order == 2)
                    myDescr = descriptors(:, myPath);
                end
                
                % find best score
                bestInd = bestInPath(S, myPath, order, myDescr);

                if (~bestInd)
                    continue;
                end

                % label it a stone
                S.labels(bestInd) = 1;

                % set children to -1
                S = setChildrenValue(S,bestInd,-1);

                % unhook children of parents and set parents to -1
                S = unhookSibs(S, bestInd, S.root, true);
            end

        % voting
        case 2

            % allocate memory for votes
            myVotes = zeros(1, numel(S.regions), 'uint16');
            
            % loop over all paths
            for i=1:n

                % extract path
                myPath = extractPath(S, i);
                
                % extract descriptors and set fields and weights (if necessary)
                if (order == 2)
                    myDescr = descriptors(:, myPath);
                    myFields = false(size(myDescr, 1), 1);
                    myWeights = zeros(size(myDescr, 1), 1);
                    % for now pick just solidity (5) and circularity (6)
                    % keep in mind that the descriptors might change
                    myFields(5) = true;
                    myFields(6) = true;
                    % weight solidity more
                    myWeights(5) = 0.8;
                    myWeights(6) = 0.2;

                end
                
                % find best score
                bestInd = bestInPath(S, myPath, order, myDescr, myFields, myWeights);

                if (~bestInd)
                    continue;
                else
                    myVotes(bestInd) = myVotes(bestInd) + 1;
                end
            end
            
            % find out how many stones got votes
            myStones = find(myVotes);
            numStones = numel(myStones);
            
            % sort the stones by votes
            [sortArr sortInd] = sort(myVotes, 'descend');
            
            % loop over all sorted stones
            n = numStones;
            for i=1:n
                
                % get index of stone
                myStone = sortInd(i);
                
                % check if it is still labeled a stone
                if (S.labels(myStone) == 1)

                    % set children to -1
                    S = setChildrenValue(S,myStone,-1);

                    % unhook children of parents and set parents to -1
                    S = unhookSibs(S, myStone, S.root, true);
                    
                end
            end

        otherwise
            error('doClassify: no such selection method!');
    end
    
    % update stones
    S = updateAllStones(S);
    toc
    
%%  results
    fprintf(['Examining results for ', baseName, ' ...\n']);
    tic
    
    % contruct label matrix
    fprintf(['Constructing label matrix for ', baseName, ' ...\n']);
    stoneMat = zeros(size(S.regions{1}), 'uint16');
    S.stoneMat = zeros(size(S.regions{1}), 'uint16');
    numStones = numel(S.stones);
    
    % stoneMat is labeled sequentially for regionpros
    % stoneMat is a labeled matrix (size of img) with labels 1:numStones
    % S.stoneMat is a labeled matrix (size of img) with labels = regNum
    for i=1:numStones   % can this be vectorized?
        stoneMat = stoneMat + uint16(S.regions{1,S.stones(i)}).*i;
        S.stoneMat = S.stoneMat + uint16(S.regions{1,S.stones(i)}).*S.stones(i);
    end
    
    mp=rand(256,3);
    transp = 0.25;

    figure;imshow(imageName);
    hold on
    mask=zeros(size(stoneMat))+transp;
    h = imshow(stoneMat,[]);colormap(mp);
    set(h,'AlphaData',mask);
    hold off
    title('Classified clasts');
    impixelinfo
    
    % measure
    fprintf(['Measuring ', baseName, ' ...\n']);
    stoneStats = regionprops(stoneMat, 'Centroid', 'MinorAxisLength', ...
        'MajorAxisLength', 'Orientation');
    S.stoneStats = stoneStats;
    toc
    
    % save information back to file
    % how should we save for the visualizer?
    % answer: just like for the GUI (save S)
    % make copy of S as S2 for gui
    fprintf(['Saving results for ', baseName, ' ...\n']);
    tic
    S2 = S;
    save(finalName, 'S2');
    save(statsName, 'stoneStats');
    toc
    
end
