function [svm_model, svm_type, svm_params, svm_opts, labels, accuracy, value] = ...
    trainSVM(baseName, params, options)
% author: dino bellugi
% copyright: blueG soft wear 2010
% trains SVM given training data (libSVM version)
% inputs:
%   baseName = base name for files
%   params = parameters for training
%       params(1): 0=load model, 1=find best parameters, 2=use defaults
%       params(2): 0=linear, 1=poly, 2=rbf, 3=sigmoid
%       params(3): minimum degree (for poly only)
%       params(4): maximum degree (for poly only)
%       params(5): number of subsets for cross-validation
%       params(6): lower bound of C search space (log)
%       params(7): upper bound of C search space (log)
%       params(8): lower bound of G search space (log)
%       params(9): upper bound of G search space (log)
%       params(10): coarse search step (in the log space)
%       params(11): fine search step (in the log space)
%   options = logical(1,3) for print, save, test
% outputs:
%   will come soon
% example:
%   [model, type, pars, opts, lab, acc, val] = trainSVM('../newData/Img0057_cropped', [1 1 10 5 -5 5], [true true true]);
%   [model, type, pars, opts, lab, acc, val] = trainSVM('../newData/Img0158_cropped', [1 1 5 4 -5 5], true(1,3))
% notes:
%   has some hard-coded parameters (see below)
%   expects libSVM in path:
%   addpath('../libSVM')
%   addpath('../libSVM/libsvm-mat-2.89-3')
% naming convention:
%   trainName = [baseName '_trainData'] 
%   modelName = [baseName '_model']
% notes:
%   <trainName> must contain variables 'trainDescriptors' and 'trainLabels'
%   <modelMame> will contain 'svm_model', 'svm_type', 'svm_params', 'svm_opts'
%   hard coded "searchStep" gives the parameter space increment

    %% check command line
    if (nargin < 2 || nargin > 3)
        error(['Usage: [svm_model, svm_type, svm_params, svm_opts, accuracy, ', ...
            'value] = trainSVM(baseName, labels, params, [options])\n']);
    end
    if (nargin < 3)
        options = true(3,1);
    end
    
    %% setup naming structure
    trainName = [baseName '_trainData.mat'];
    modelName = [baseName '_model_' num2str(params(2))];
    coarseName = [modelName '_coarse'];
    fineName = [modelName '_fine'];

    %% load and check data
    load(trainName);
    if (size(trainDescriptors, 2) ~= size(trainLabels, 2)) %#ok<NODEF>
        error('trainSVM error: labels and descriptors do not match!\n');
    end
    
    trainDescriptors = trainDescriptors';
    trainLabels = trainLabels';

    % brutally replace NaN with 0's
    trainDescriptors(isnan(trainDescriptors)) = 0;

    % type, options, automate
    svm_mode = params(1);	 % 0 = load, 1 = search, 2 = default
    ker_type = params(2);    % 0 = linear, 1 = poly, etc.
    min_deg = params(3);     % max degree of kernel
    max_deg = params(4);     % max degree of kernel
    sub_num = params(5);     % number of subsets for cross validation
    min_csearch = params(6); % lower bound of C search space (log)
    max_csearch = params(7); % upper bound of C search space (log)
    min_gsearch = params(8); % lower bound of G search space (log)
    max_gsearch = params(9); % upper bound of G search space (log)
    coarse_step = params(10); % coarse cost and gamma increments for search
    fine_step = params(11);   % coarse cost and gamma increments for search
    num_deg = max_deg - min_deg + 1;
        
    % define default parameters
    % -s svm_type : set type of SVM (default 0)
    %         0 -- C-SVC
    %         1 -- nu-SVC
    %         2 -- one-class SVM
    %         3 -- epsilon-SVR
    %         4 -- nu-SVR
    % -t kernel_type : set type of kernel function (default 2)
    %         0 -- linear: u'*v
    %         1 -- polynomial: (gamma*u'*v + coef0)^degree
    %         2 -- radial basis function: exp(-gamma*|u-v|^2)
    %         3 -- sigmoid: tanh(gamma*u'*v + coef0)
    %         4 -- precomputed kernel (kernel values in training_set_file)
    % -d degree : set degree in kernel function (default 3)
    % -g gamma : set gamma in kernel function (default 1/k)
    % -r coef0 : set coef0 in kernel function (default 0)
    % -c cost : set the parameter C of C-SVC, epsilon-SVR, 
    %   and nu-SVR (default 1)
    % -n nu : set the parameter nu of nu-SVC, one-class SVM, 
    %   and nu-SVR (default 0.5)
    % -p epsilon : set the epsilon in loss function of epsilon-SVR (default 0.1)
    % -m cachesize : set cache memory size in MB (default 100)
    % -e epsilon : set tolerance of termination criterion (default 0.001)
    % -h shrinking : whether to use the shrinking heuristics, 0 or 1 (default 1)
    % -b probability_estimates : whether to train a SVC or SVR 
    %   model for probability estimates, 0 or 1 (default 0)
    % -wi weight : set the parameter C of class i to weight*C, 
    %   for C-SVC (default 1)
    % -v n: n-fold cross validation mode
    % -q : quiet mode (no outputs)

    ker_opts = [' -d ' num2str(max_deg) ' '];
    svm_type = [' -t ' num2str(ker_type) ' '];   % svm type, etc.
    svm_opts = ' -b 1 ';              % quiet mode, probabilities, etc.

    %% get model parameters
    % set cost and gamma parameters
 
    % load pre-existing model
    if (svm_mode == 0)
        if (options(1))
            fprintf(['Loading SVM model ', modelName, ' ...\n']);
        end
        load(modelName, 'svm_model', 'svm_type', 'svm_params', 'svm_opts');   
    
    % search paramater space for best model using cross-correlation
    elseif (svm_mode == 1)
        if (options(1))
            fprintf('Searching for best SVM parameters ...\n');
        end
        tTotal = tic;
        bestcv = 0;
        bestc = 0;
        bestg = 0;
        bestIter = 0;
        bestdeg = 0;
        my_deg = 0;
 
        % polynomial kernel case
        % needs to be modified as in the other kernels to do the coarse
        % then fine searches with user input
        if (ker_type == 1)
                       
            % do a coarse search
            hWB = waitbar(0, ['Searching parameter space (coarse) for degrees ' num2str(min_deg) ' to ' num2str(max_deg) ' ...']);
            numIters = 1;
            numCIters = (max_csearch - min_csearch) / coarse_step + 1;
            numGIters = (max_gsearch - min_gsearch) / coarse_step + 1;
            totIters = numCIters * numGIters * num_deg;
            cv = zeros(ceil(numGIters), ceil(numCIters), num_deg);
            xvec = min_csearch:coarse_step:max_csearch;
            yvec = min_gsearch:coarse_step:max_gsearch;
            [xcv,ycv] = meshgrid(xvec, yvec);
                
            % loop over poly degree
            for deg = 1:num_deg
                my_deg = min_deg + deg - 1;
                waitbar(numIters/totIters, hWB, ['Searching parameter space (coarse) for degree ' num2str(my_deg) ' ...']);
                ker_opts = [' -d ', num2str(my_deg), ' '];
                tStart = tic;
                
                % loop over cost parameter
                for indlog2c = 1:numel(xvec)
                    log2c = xvec(indlog2c);

                    % loop over gamma parameter
                    for indlog2g = 1:numel(yvec)
                        log2g = yvec(indlog2g);
                        
                        if (options(1))
                            fprintf('iter = %g, deg=%g ln(c)=%g ln(g)=%g\n', ...
                                numIters, my_deg, log2c, log2g);
                        end
                        
                        svm_cmd = ['-v ', num2str(sub_num), ' -c ', ...
                            num2str(2^log2c), ' -g ', num2str(2^log2g)];
                        tic
                        cv(indlog2g, indlog2c, deg) = svmtrain(trainLabels, trainDescriptors, [svm_cmd svm_type svm_opts ker_opts]);
                        toc
                        
                        numIters = numIters+1;
                        waitbar(numIters/totIters, hWB);
                        
                        if (cv(indlog2g, indlog2c, deg) > bestcv)
                            bestdeg = my_deg; bestcv = cv(indlog2g, indlog2c, deg); 
                            bestc = 2^log2c; bestg = 2^log2g; bestIter = numIters;
                        end
                        
                        if (options(1))
                            fprintf('x-val=%g best iter = %g, best deg=%g, best c=%g, g=%g, rate=%g)\n', ...
                                cv(indlog2g, indlog2c, deg), bestIter, bestdeg, bestc, bestg, bestcv);
                        end
                    end
                end
                
                % elapsed time for this degree
                fprintf('elapsed time for coarse search on degree %d: %f seconds\n' , my_deg, toc(tStart));
                
                % save coarse data for current degree
                save(coarseName, 'cv', 'xvec', 'yvec', 'my_deg', 'bestIter', 'bestdeg', 'bestc', 'bestg', 'bestcv');
                
                % make coarse figure for current degree
                figure; contourf(xcv, ycv, cv(:,:,deg)); colorbar; title(['Course parameter space (degree ' num2str(my_deg) ')']); xlabel('log2c'); ylabel('log2g');
                
            end

            close(hWB);

            % save all coarse data
            save(coarseName, 'cv', 'xvec', 'yvec', 'min_deg', 'max_deg', 'bestIter', 'bestdeg', 'bestc', 'bestg', 'bestcv');
            
            % ask the users for degree and min and max search parameters
            range = input('Input degree min-C max-C min-G max-G: ', 's');
            range = sscanf(range, '%f');
            deg = range(1);
            min_csearch2 = range(2);
            max_csearch2 = range(3);
            min_gsearch2 = range(4);
            max_gsearch2 = range(5);

            % do a fine search
            hWB = waitbar(0, ['Searching parameter space (fine) for degree ' num2str(deg) ' ...']);
            tStart = tic;
            numIters = 1;
            numCIters = (max_csearch2 - min_csearch2) / fine_step + 1;
            numGIters = (max_gsearch2 - min_gsearch2) / fine_step + 1;
            totIters = numCIters * numGIters;
            cv = zeros(ceil(numGIters), ceil(numCIters));
            xvec = min_csearch2:fine_step:max_csearch2;
            yvec = min_gsearch2:fine_step:max_gsearch2;
            [xcv,ycv] = meshgrid(xvec, yvec);

            % loop over cost parameter
            for indlog2c = 1:numel(xvec)
                log2c = xvec(indlog2c);

                % loop over gamma parameter
                for indlog2g = 1:numel(yvec)
                    log2g = yvec(indlog2g);

                    if (options(1))
                        fprintf('iter - %g, ln(c)=%g ln(g)=%g\n', numIters, log2c, log2g);
                    end

                    svm_cmd = ['-v ', num2str(sub_num), ' -c ', ...
                        num2str(2^log2c), ' -g ', num2str(2^log2g)];

                    tic
                    cv(indlog2g, indlog2c) = svmtrain(trainLabels, trainDescriptors, [svm_cmd svm_type svm_opts ker_opts]);
                    toc

                    numIters = numIters+1;
                    waitbar(numIters/totIters, hWB);

                    if (cv(indlog2g, indlog2c) > bestcv)
                       bestcv = cv(indlog2g, indlog2c); bestc = 2^log2c; bestg = 2^log2g; bestIter = numIters;
                    end

                    if (options(1))
                        fprintf('x-val=%g best iter=%g, best c=%g, g=%g, rate=%g)\n', ...
                            cv(indlog2g, indlog2c), bestIter, bestc, bestg, bestcv);
                    end
                end
            end

            close(hWB);

            % elapsed time for fine search
            fprintf('elapsed time for fine search on degree %d: %f seconds\n' , my_deg, toc(tStart));

            % save fine data
            save(fineName, 'cv', 'xvec', 'yvec', ' deg', 'bestIter', 'bestdeg', 'bestc', 'bestg', 'bestcv');
            figure; contourf(xcv, ycv, cv); colorbar; title(['Fine parameter space (degree ' num2str(deg) ')']); xlabel('log2c'); ylabel('log2g');
            
            % save best parameters
            ker_opts = [' -d ', num2str(bestdeg), ' '];
            svm_params = [ker_opts, ' -c ', num2str(bestc), ' -g ', num2str(bestg)];
                    
        % other kernels
        else
            % do a coarse search
            hWB = waitbar(0, 'Searching parameter space (coarse search)...');
            tStart = tic;
            numIters = 1;
            numCIters = (max_csearch - min_csearch) / coarse_step + 1;
            numGIters = (max_gsearch - min_gsearch) / coarse_step + 1;
            totIters = numCIters * numGIters;
            cv = zeros(ceil(numGIters), ceil(numCIters));
            xvec = min_csearch:coarse_step:max_csearch;
            yvec = min_gsearch:coarse_step:max_gsearch;
            [xcv,ycv] = meshgrid(xvec, yvec);
            
            % loop over cost parameter
            for indlog2c = 1:numel(xvec)
                log2c = xvec(indlog2c);
                
                % loop over gamma parameter
                for indlog2g = 1:numel(yvec)
                    log2g = yvec(indlog2g);
 
                    if (options(1))
                        fprintf('iter - %g, ln(c)=%g ln(g)=%g\n', numIters, log2c, log2g);
                    end
                    
                    svm_cmd = ['-v ', num2str(sub_num), ' -c ', ...
                        num2str(2^log2c), ' -g ', num2str(2^log2g)];
                    
                    tic
                    cv(indlog2g, indlog2c) = svmtrain(trainLabels, trainDescriptors, [svm_cmd svm_type svm_opts]);
                    toc
                    
                    waitbar(numIters/totIters, hWB);
                    
                    if (cv(indlog2g, indlog2c) > bestcv)
                       bestcv = cv(indlog2g, indlog2c); bestc = 2^log2c; bestg = 2^log2g; bestIter = numIters;
                    end
                    
                    if (options(1))
                        fprintf('x-val=%g best iter=%g, best c=%g, g=%g, rate=%g)\n', ...
                            cv(indlog2g, indlog2c), bestIter, bestc, bestg, bestcv);
                    end
                    numIters = numIters+1;
                end
            end
            
            close(hWB);
            
            % elapsed time for coarse search
            fprintf('elapsed time for coarse search: %f seconds\n' , toc(tStart));
            
            % save coarse data
            save(coarseName, 'cv', 'xvec', 'yvec', 'bestIter', 'bestc', 'bestg', 'bestcv');

            % coarse figure
            figure; contourf(xcv, ycv, cv); colorbar; title('Course parameter space'); xlabel('log2c'); ylabel('log2g');

            % ask the users for min and max search parameters
            range = input('Input min-C max-C min-G max-G: ', 's');
            range = sscanf(range, '%f');
            min_csearch2 = range(1);
            max_csearch2 = range(2);
            min_gsearch2 = range(3);
            max_gsearch2 = range(4);
            
            % do a fine search
            hWB = waitbar(0, 'Searching parameter space (fine search)...');
            tStart = tic;
            numIters = 1;
            numCIters = (max_csearch2 - min_csearch2) / fine_step + 1;
            numGIters = (max_gsearch2 - min_gsearch2) / fine_step + 1;
            totIters = numCIters * numGIters;
            cv = zeros(ceil(numGIters), ceil(numCIters));
            xvec = min_csearch2:fine_step:max_csearch2;
            yvec = min_gsearch2:fine_step:max_gsearch2;
            [xcv,ycv] = meshgrid(xvec, yvec);
            
            % loop over cost parameter
            for indlog2c = 1:numel(xvec)
                log2c = xvec(indlog2c);
                
                % loop over gamma parameter
                for indlog2g = 1:numel(yvec)
                    log2g = yvec(indlog2g);

                    if (options(1))
                        fprintf('iter - %g, ln(c)=%g ln(g)=%g\n', numIters, log2c, log2g);
                    end
                    
                    svm_cmd = ['-v ', num2str(sub_num), ' -c ', ...
                        num2str(2^log2c), ' -g ', num2str(2^log2g)];
                    
                    tic
                    cv(indlog2g, indlog2c) = svmtrain(trainLabels, trainDescriptors, [svm_cmd svm_type svm_opts]);
                    toc
                    
                    numIters = numIters+1;
                    waitbar(numIters/totIters, hWB);
                    
                    if (cv(indlog2g, indlog2c) > bestcv)
                       bestcv = cv(indlog2g, indlog2c); bestc = 2^log2c; bestg = 2^log2g; bestIter = numIters;
                    end
                    
                    if (options(1))
                        fprintf('x-val=%g best iter=%g, best c=%g, g=%g, rate=%g)\n', ...
                            cv(indlog2g, indlog2c), bestIter, bestc, bestg, bestcv);
                    end
                end
            end
            
            close(hWB);
            
            % elapsed time for fine search
            fprintf('elapsed time for fine search: %f seconds\n' , toc(tStart));

            % save fine data
            save(fineName, 'cv', 'xvec', 'yvec', 'bestIter', 'bestc', 'bestg', 'bestcv');
            
            % fine figure
            figure; contourf(xcv, ycv, cv); colorbar; title('Fine parameter space'); xlabel('log2c'); ylabel('log2g');

            % save best parameters
            svm_params = [' -c ', num2str(bestc), ' -g ', num2str(bestg) ' '];
        end
        toc(tTotal)
    
    % use default parameters (currently not set)
    else
        if (options(1))
            fprintf('Using default SVM parameters ...\n');
        end
        % define here or leave blank for defaults
        svm_params = '';
    end

    %% train
    if (options(1))
        fprintf('Training SVM ...\n');
    end
    tic
    svm_model = svmtrain(trainLabels, trainDescriptors, [svm_type svm_params svm_opts]);
    toc

    %% save
    if (options(2))
        if (options(1))
            fprintf('Saving SVM ...\n');
        end
        save(modelName, 'svm_model', 'svm_type', 'svm_params', 'svm_opts');
    end

    %% test
    if (options(3))
        if (options(1))
            fprintf('Testing SVM ...\n');
        end
        tic
        [labels, accuracy, value] = svmpredict(trainLabels, trainDescriptors, svm_model, svm_opts);
        toc
    end

end