function trainsvmGUI(baseName, mode)
%
% This is a GUI interface designed to help gather training data for the
% grain-size identification SVM. The GUI loads the image, which has already
% undergone analysis for gPb, UCM, its associated region tree. The user
% then navigates the image and labels regions as stones; this automatically
% then labels that region's parents and children as notstone. 
%
% baseName is a string specifying the file naming structure used for the
% image.
%
% mode is a logical indicating if the GUI is to be used in training (true)
% or visualizing (false) mode.
%
% The user
% should periodically save their work to a .mat file by clicking the 'Save'
% button. This will save structure S2 to the .mat file. S2 contains all of the
% labeling information. It also saves regList and labels, which should be
% of the format used in createDescriptors.m and trainSVM.m.
% 
% The relevant outputs for the SVM will be S2.stones
% and S2.notstones - these are row vectors containing the region ID's of
% regions that have been labeled as positive or negative regions.
%
% The "Load" button can be used to load the label data from the specified
% mat file. This button assumes that the GUI has already been loaded with
% the appropriate baseName files (images, parent, regions, etc.); all the
% "load" button does is load some previously saved S2 data structure so the
% user can continue labeling where they left off.
%
% Author: Peter Nelson
% Date: 04/03/2010

%% set up structure of gui

% Initialization tasks
% close all
% clear all

% check command line
if (nargin < 1 || nargin > 2)
    error('Usage: trainsvmGUI(baseName, [mode])');
end

% set default mode
if (nargin < 2)
    mode = true;
end

% add paths
addLibs();

% file names
imageName = [baseName '.bmp'];
labelName = [baseName '_labels.mat'];

% init structure
S2 = initS(baseName);

% quick and dirty stuff for visualization mode
if (~mode)
    loaddata = load(labelName, 'S2');
    S2 = loaddata.S2;
    labelName = [baseName '_finalS.mat'];
    loaddata = load(labelName);
    S2.labels = loaddata.S2.labels;
    S2.value = loaddata.S2.value;
    S2.accuracy = loaddata.S2.accuracy;
    S2.stones = loaddata.S2.stones;
    S2.notstones = loaddata.S2.notstones;
    clear loaddata;  
    save(labelName);
end

% ------------------------------------
% Construct the components of the GUI

% figure, axes, and panels:
S.fh = figure('Name','Training GUI','Position',[70 57 1094 692]);
S.ah1 = axes('Parent',S.fh,'Position',[.05 .15 0.4 0.7]);
S.ah2 = axes('Parent',S.fh,'Position',[0.55 0.58 0.4 0.4]);
S.ph = uipanel('Parent',S.fh,'Title','Navigation','Position',[.55 .15 .4 .4]);
S.ph2 = uipanel('Parent',S.fh,'Title','Saving','Position',[.55 .05 .4 .1]);

% pushbuttons for zooming
S.pbhzin = uicontrol(S.fh,'Style','pushbutton','String','Zoom in',...
    'Units','normalized','Position',[.05 .05 .13 .05]);
S.pbhzout = uicontrol(S.fh,'Style','pushbutton','String','Zoom out',...
    'Units','normalized','Position',[.18 .05 .13 .05]);
S.pbhreset = uicontrol(S.fh,'Style','pushbutton','String','Reset',...
    'Units','normalized','Position',[.31 .05 .13 .05]);

% pushbuttons for navigation
S.pbhup = uicontrol(S.ph,'Style','pushbutton','String','Go to Parent (up)',...
    'Units','normalized','Position',[.35 .5 .3 .2]);
S.pbhdn = uicontrol(S.ph,'Style','pushbutton','String','Go to Child (down)',...
    'Units','normalized','Position',[.35 .25 .3 .2]);
S.pbhl = uicontrol(S.ph,'Style','pushbutton','String','Go to Sibling (left)',...
    'Units','normalized','Position',[.03 .375 .3 .2]);
S.pbhr = uicontrol(S.ph,'Style','pushbutton','String','Go to Sibling (right)',...
    'Units','normalized','Position',[.67 .375 .3 .2]);

% pushbuttons for marking and skipping to next leaf
S.pbhch = uicontrol(S.ph,'Style','pushbutton','String','Mark as Stone (y)',...
    'Units','normalized','Position',[.03 .02 .3 .2]);
S.pbhnch = uicontrol(S.ph,'Style','pushbutton','String','Mark as notStone (n)',...
    'Units','normalized','Position',[.35 .02 .3 .2]);
S.pbhnx = uicontrol(S.ph,'Style','pushbutton','String','Skip to next leaf (\)',...
    'Units','normalized','Position',[.67 .02 .3 .2]);

% user-selected region
S.sth1 = uicontrol(S.ph,'Style','text','String','Currently selected region:',...
    'Units','normalized','Position',[.02 0.85 0.4 0.06]);
S.eth1 = uicontrol(S.ph,'Style','edit','String',num2str(S2.current),...
    'Units','normalized','Position',[.4 .85 .1 .1]);
S.pbhgo = uicontrol(S.ph,'Style','pushbutton','String','Go',...
    'Units','normalized','Position',[.5 .85 .1 .1]);

% info about the region (scale, # of siblings, label)
S.sth2 = uicontrol(S.ph,'Style','text','String',strcat('Level: ',num2str(S2.scale(S2.current))),...
    'Units','normalized','Position',[.6 0.9 0.4 0.06]);
S.sth3 = uicontrol(S.ph,'Style','text','String',strcat('Number of siblings: ',...
    num2str(length(S2.siblings{S2.current}))),...
    'Units','normalized','Position',[.02 .8 .4 .06]);
S.sth4 = uicontrol(S.ph,'Style','text','String','Label: unassigned',...
    'Units','normalized','Position',[.6 0.85 0.4 0.06]);
S.sth7 = uicontrol(S.ph,'Style','text','String','0 positive regions',...
    'Units','normalized','Position',[.6 .8 .4 .06]);
S.sth8 = uicontrol(S.ph,'Style','text','String','0 negative regions',...
    'Units','normalized','Position',[.6 .75 .4 .06]);

% checkboxes for plotting and image name
S.chb1 = uicontrol(S.fh,'Style','checkbox',...
    'String','Show children (green outline)',...
    'Units','normalized','Position',[.05 .95 .2 .04],...
    'Value',1);
S.chb2 = uicontrol(S.fh,'Style','checkbox',...
    'String','Show parent (red outline)',...
    'Units','normalized','Position',[.05 .92 .2 .04],...
    'Value',1);
S.chb3 = uicontrol(S.fh,'Style','checkbox',...
    'String','Show siblings (white outline)',...
    'Units','normalized','Position',[.05 .89 .2 .04],...
    'Value',1);
S.sth6 = uicontrol(S.fh,'Style','text','String',imageName,...
    'Units','normalized','Position',[.05 .85 .4 .03]);
S.chb4 = uicontrol(S.fh,'Style','checkbox',...
    'String','Shade selected region (yellow)',...
    'Units','normalized','Position',[.25 .95 .2 .04],...
    'Value',1);
S.chb5 = uicontrol(S.fh,'Style','checkbox',...
    'String','Shade siblings (blue)',...
    'Units','normalized','Position',[.25 .92 .2 .04],...
    'Value',1);
S.chb6 = uicontrol(S.fh,'Style','checkbox',...
    'String','Shade positive regions (white)',...
    'Units','normalized','Position',[.25 .89 .2 .04],...
    'Value',1);

% stuff for saving the labels
S.pbhsv = uicontrol(S.ph2,'Style','pushbutton','String','Save',...
    'Units','normalized','Position',[.05 .05 .2 .8]);
% S.sth5 = uicontrol(S.ph2,'Style','text','String','label filename:',...
%     'Units','normalized','Position',[.27 .05 .2 .6]);
S.eth2 = uicontrol(S.ph2,'Style','edit','String',labelName,...
    'Units','normalized','Position',[.47 .1 .5 .8]);
S.pbhload = uicontrol(S.ph2,'Style','pushbutton','String','Load',...
    'Units','normalized','Position',[.26 .05 .2 .8]);

% set the figure background color to be the same as the defaul uicontrol
% background color
defaultBackground = get(0,'defaultUicontrolBackgroundColor');
set(S.fh,'Color',defaultBackground)

%%

setappdata(S.fh,'mydata',S2); %save labels, etc. for access in callbacks
updatecallbacks(S); %initially sets all of the callbacks
pbreset_call([],[],S)


% ------------------------------------------
% Callbacks

    function [] = pbup_call(varargin)
        % switch to the parent if up button pressed
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        ind = S2.current;
        S2.current = getParent(S2,ind);
        if S2.current == 0 %top region
            S2.current  = ind;
        end
        setappdata(S.fh,'mydata',S2);
        pbreset_call([],[],S)
    end

    function [] = pbgo_call(varargin)
        % go to user defined node
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        ind = str2num(get(S.eth1,'String'));
        S2.current = ind;
        setappdata(S.fh,'mydata',S2);
        pbreset_call([],[],S)
    end

    function [] = pbdn_call(varargin)
        % go to the child (lowest ID)
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        [children nKids] = getChildren(S2,S2.current);
        if nKids > 0
            if getParent(S2,S2.current) == 0
                S2.current = children(2); %first child of whole image is itself
            else
                S2.current = children(1);
            end
            setappdata(S.fh,'mydata',S2);
            pbreset_call([],[],S)
        end
    end

    function [] = pbr_call(varargin)
        % go to sibling (increasing order)
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        [sibs nSibs] = getSibs(S2,S2.current);
        if nSibs > 1
            if S2.current > max(sibs) %loop around to first sibling on list
                nextsib = min(sibs);
            else
                nextsib = min(sibs(find(sibs > S2.current)));
            end
        else
            nextsib = sibs(1);
        end
        S2.current = nextsib;
        
        % get new centroids
        regstats = regionprops(bwlabel(S2.regions{S2.current}),'Centroid');
        S2.xc = regstats.Centroid(1);
        S2.yc = regstats.Centroid(2);
        
        setappdata(S.fh,'mydata',S2);
        updatevisible(S)
    end

    function [] = pbl_call(varargin)
        % go to sibling (decreasing order)
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        [sibs nSibs] = getSibs(S2,S2.current);
        if nSibs > 1
            if S2.current < min(sibs) %loop around to last sibling on list
                nextsib = max(sibs);
            else
                nextsib = max(sibs(find(sibs < S2.current)));
            end
        else
            nextsib = sibs(1);
        end
        S2.current = nextsib;
        
        % get new centroids
        regstats = regionprops(bwlabel(S2.regions{S2.current}),'Centroid');
        S2.xc = regstats.Centroid(1);
        S2.yc = regstats.Centroid(2);
        
        setappdata(S.fh,'mydata',S2);
        updatevisible(S)
    end

    function [] = pbnx_call(varargin)
        % go to next leaf
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        % find the next leaf that hasn't been labeled yet
        S2 = gotoNextLeaf(S2);

        setappdata(S.fh,'mydata',S2); %save the incremented counter
        pbreset_call([],[],S)
    end

    function [] = chb_call(varargin)
        % replot when a checkbox is changed
        S = varargin{3};
        updatevisible(S);
    end

    function [] = pbch_call(varargin)
        % mark region as stone, and other regions in its tree as notstone
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        ind = S2.current;
        if S2.labels(ind) == 0 %not yet labeled
            S2.labels(ind) = 1; % mark as stone
            S2 = setChildrenValue(S2,ind,-1); %set all children to -1
            S2 = setParentsValue(S2,ind,-1); %set all parents to -1
%         elseif S2.labels(ind) == 1 %already labeled as rock (do nothing)
%             return
        elseif S2.labels(ind) == -1 %previously labeled as norock
            qstring = 'Region has previously been labeled as notstone. Switch label to stone?';
            qtitle = 'Are you sure?';
            button = questdlg(qstring,qtitle,'Yes','No','No');
            switch button
                case 'Yes'
%             if button == 'Yes'
                % check to see if a parent had been labeled stone; if yes,
                % switch it to notstone and mark it, and make all of its
                % children unmarked
                S2 = switchParentsValue(S2,ind,-1,1,0);
                % now mark all parents as notstone, mark the region as
                % stone, and mark all children as notstone
                S2 = setParentsValue(S2,ind,-1);
                S2.labels(ind) = 1;
                S2 = setChildrenValue(S2,ind,-1);
            end
        end
        
        % update S2.stones and S2.notstones
        S2 = updateAllStones(S2);

        % if not yet labeled, make the stone's next sibling the active
        % region. if that is already labeled, go to the next available leaf
        [mySibs nSibs] = getSibs(S2,ind);
        gotit = 0;
        for k=1:nSibs
            if S2.labels(mySibs(k)) == 0
                S2.current = mySibs(k);
                setappdata(S.fh,'mydata',S2);
                pbreset_call([],[],S)
                gotit = 1;
                break
            end
        end
        if gotit == 0 %all siblings labeled already
            setappdata(S.fh,'mydata',S2);
            pbnx_call([],[],S);
        end
    end

    function [] = pbnch_call(varargin)
        % mark region as notstone (used for leaves that aren't stones).
        % After marking, go to sibling; if labeled, go to parent; if
        % labeled as stone, go to next leaf.
        % if switching from stone to notstone, make all children unlabeled,
        % and go up the list of parents and mark unlabeled so long as they
        % don't have other children already labeled as stones.
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        ind = S2.current;
        if S2.labels(ind) == 0
            S2.labels(ind) = -1;
        elseif S2.labels(ind) == 1 %already marked as stone
            qstring = 'Region has previously been labeled as stone. Switch label to notstone?';
            qtitle = 'Are you sure?';
            button = questdlg(qstring,qtitle,'Yes','No','No');
            switch button
                case 'Yes'
                    S2 = setChildrenValue(S2,ind,0);
                    S2.labels(ind) = -1;
                    finished = 0;
                    indt = ind;
                    while finished == 0
                        [mySibs nSibs] = getSibs(S2,indt);
                        for i=1:nSibs
                            if S2.labels(mySibs(i)) ~= 0
                                S2 = setParentsValue(S2,mySibs(i),-1);
                                finished = 1;
                                break
                            end
                        end
                        if getParent(S2,indt) == 0
                            finished = 1;
                        else
                            indt = getParent(S2,indt);
                            S2.labels(indt) = 0;
                        end
                    end
            end
        end

        % update S2.stones and S2.notstones
        S2 = updateAllStones(S2);

        % if not yet labeled, make the stone's next sibling the active
        % region. if that is already labeled, go to the next available leaf
        [mySibs nSibs] = getSibs(S2,ind);
        gotit = 0;
        for k=1:nSibs
            if S2.labels(mySibs(k)) == 0
                S2.current = mySibs(k);
                setappdata(S.fh,'mydata',S2);
                pbreset_call([],[],S)
                gotit = 1;
                break
            end
        end
        if gotit == 0 %all siblings labeled already
            parentind = getParent(S2,ind);
            if S2.labels(parentind) == 0
                S2.current = parentind;
                setappdata(S.fh,'mydata',S2);
            else
                setappdata(S.fh,'mydata',S2);
                pbnx_call([],[],S);
            end
        end
    end
        

    function [] = pbsv_call(varargin)
        % save label data to mat file
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        savefilename = get(S.eth2,'String');
        % make regList and labels for use in createDescriptors.m and
        % trainSVM.m. regList is a row vector with region id's, and labels
        % is a row vector the same size as regList where 1 corresponds to
        % stone, -1 corresponds to not stone, and 0 corresponds to notstone.
        regList = [1:length(S2.labels)];
        labels = S2.labels;
        nonlabeledinds = find(labels == 0);
        regList(nonlabeledinds) = [];
        labels(nonlabeledinds) = [];
        clear varargin
        save(savefilename,'S2','regList','labels');
    end

    function [] = pbload_call(varargin)
        % load data from mat file
        S = varargin{3};
        loadfilename = get(S.eth2,'String');
        loaddata = load(loadfilename);
        S2 = loaddata.S2;
        setappdata(S.fh,'mydata',S2)
        pbreset_call([],[],S)
        updatevisible(S);
    end

    function [] = pbzin_call(varargin)
        % zoom in, centering on selected region
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        
        xl = 0.9*(S2.xmax - S2.xmin);
        yl = 0.9*(S2.ymax - S2.ymin);
        
        [r,c] = size(S2.leafRegNum); %whole image dimensions
        S2.xmin = max(S2.xc - xl/2, 0);
        S2.xmax = min(S2.xc + xl/2, c);
        S2.ymin = max(S2.yc - yl/2, 0);
        S2.ymax = min(S2.yc + yl/2, r);

        setappdata(S.fh,'mydata',S2);
        updatevisible(S)
    end

    function [] = pbzout_call(varargin)
        % zoom out
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        
        xl = S2.xmax - S2.xmin;
        yl = S2.ymax - S2.ymin;
        [r,c] = size(S2.leafRegNum); %whole image dimensions
        S2.xmin = max(S2.xmin-0.1*xl, 0);
        S2.xmax = min(S2.xmax+0.1*xl, c);
        S2.ymin = max(S2.ymin-0.1*yl, 0);
        S2.ymax = min(S2.ymax+0.1*yl, r);
        
        setappdata(S.fh,'mydata',S2);
        updatevisible(S)
    end

    function [] = pbreset_call(varargin)
        % reset zoom to show whole parent region with a slight 
        S = varargin{3};
        S2 = getappdata(S.fh,'mydata');
        ind = S2.current;
        par = S2.parent(ind);
        parreg = uint16(S2.regions{par});
        B = bwboundaries(parreg);
        bound = [];
        for kk=1:length(B)
            bound = [bound; B{kk}];
        end
        xmin = min(bound(:,2));
        ymin = min(bound(:,1));
        xmax = max(bound(:,2));
        ymax = max(bound(:,1));
        
        xl = xmax-xmin;
        yl = ymax-ymin;
        
        regstats = regionprops(bwlabel(S2.regions{ind}),'Centroid');
        xc = regstats.Centroid(1);
        yc = regstats.Centroid(2);
        
        [r,c] = size(S2.leafRegNum); %whole image dimensions
        S2.xmin = max(xmin-0.1*xl, 0);
        S2.xmax = min(xmax+0.1*xl, c);
        S2.ymin = max(ymin-0.1*yl, 0);
        S2.ymax = min(ymax+0.1*yl, r);
        S2.xc = xc;
        S2.yc = yc;
        
        setappdata(S.fh,'mydata',S2);
        updatevisible(S)
    end
            

    function [] = keyread(varargin)
        S = varargin{3};
        evnt = varargin{2};

        switch evnt.Key 
            case 'uparrow'
                pbup_call([],[],S);
            case 'downarrow'
                pbdn_call([],[],S);
            case 'rightarrow'
                pbr_call([],[],S);
            case 'leftarrow'
                pbl_call([],[],S);
            case 'backslash'
                pbnx_call([],[],S);
            case 'y'
                pbch_call([],[],S);
            case 'n'
                pbnch_call([],[],S);
        end
    end

    function [] = keyread2(varargin)
        % this is only for the editable text box where the user can choose
        % to jump to a region - don't want to be navigating when left and
        % right arrows are used for moving the cursor.
        S = varargin{3};
        evnt = varargin{2};
        
        switch evnt.Key
            case 'return'
                pbgo_call([],[],S);
        end
    end




% ---------------------------------------
% Utility functions

    function updatecallbacks(S)
        %set all of the callbacks (only called at the beginning of the
        %program)
        set(S.pbhup,'Callback',{@pbup_call,S});
        set(S.pbhgo,'Callback',{@pbgo_call,S});
        set(S.pbhdn,'Callback',{@pbdn_call,S});
        set(S.pbhr,'Callback',{@pbr_call,S});
        set(S.pbhl,'Callback',{@pbl_call,S});
        set(S.pbhnx,'Callback',{@pbnx_call,S});
        set(S.fh,'KeyPressFcn',{@keyread,S});
        set(S.pbhup,'KeyPressFcn',{@keyread,S});
        set(S.pbhdn,'KeyPressFcn',{@keyread,S});
        set(S.pbhl,'KeyPressFcn',{@keyread,S});
        set(S.pbhr,'KeyPressFcn',{@keyread,S});
        set(S.pbhgo,'KeyPressFcn',{@keyread,S});
        set(S.pbhch,'KeyPressFcn',{@keyread,S});
        set(S.pbhnch,'KeyPressFcn',{@keyread,S});
        set(S.pbhnx,'KeyPressFcn',{@keyread,S});
        set(S.pbhsv,'KeyPressFcn',{@keyread,S});
        set(S.pbhload,'KeyPressFcn',{@keyread,S});
        set(S.pbhzin,'KeyPressFcn',{@keyread,S});
        set(S.pbhzout,'KeyPressFcn',{@keyread,S});
        set(S.pbhreset,'KeyPressFcn',{@keyread,S});
        set(S.chb1,'KeyPressFcn',{@keyread,S});
        set(S.chb2,'KeyPressFcn',{@keyread,S});
        set(S.chb3,'KeyPressFcn',{@keyread,S});
        set(S.chb4,'KeyPressFcn',{@keyread,S});
        set(S.chb5,'KeyPressFcn',{@keyread,S});
        set(S.chb6,'KeyPressFcn',{@keyread,S});
        set(S.eth1,'KeyPressFcn',{@keyread2,S});
        set(S.chb1,'Callback',{@chb_call,S});
        set(S.chb2,'Callback',{@chb_call,S});
        set(S.chb3,'Callback',{@chb_call,S});
        set(S.chb4,'Callback',{@chb_call,S});
        set(S.chb5,'Callback',{@chb_call,S});
        set(S.chb6,'Callback',{@chb_call,S});
        set(S.pbhch,'Callback',{@pbch_call,S});
        set(S.pbhnch,'Callback',{@pbnch_call,S});
        set(S.pbhsv,'Callback',{@pbsv_call,S});
        set(S.pbhload,'Callback',{@pbload_call,S});
        set(S.pbhzin,'Callback',{@pbzin_call,S});
        set(S.pbhzout,'Callback',{@pbzout_call,S});
        set(S.pbhreset,'Callback',{@pbreset_call,S});
    end

    function showwholeimage(S,xminp,xmaxp,yminp,ymaxp,mp,ind)
        % called by showzoomedimage, which determines the bounding box
        % coordinates
        axes(S.ah2)
        imshow(imageName)
        hold on
        %show finest leaves pixelinfo
        ihn(1) = imshow(S2.leafRegNum,[]);
        mask1 = zeros(size(S2.leafRegNum));
        set(ihn(1),'AlphaData',mask1);
        impixelinfo
        %show currently selected region
        cur_reg = uint16(S2.regions{ind});
        mask = S2.regions{ind} .* 0.3;
        ihn(2) = imshow(cur_reg,[1 0 0]);
        colormap(mp);
        set(ihn(2),'AlphaData',mask);
        %draw bounding box
        plot([xminp xminp xmaxp xmaxp xminp],[yminp ymaxp ymaxp yminp yminp],'-r')
        hold off
    end

    function showzoomedimage(S)
        S2 = getappdata(S.fh,'mydata');
        ind = S2.current;
        axes(S.ah1)
        imshow(imageName)
        hold on
        parentind = S2.parent(ind);
        cur_reg = uint16(S2.regions{ind}); %current region  = 1
        par_reg = uint16(S2.regions{parentind}).*2; %parent region = 2
        
        % shade regions if applicable boxes are checked
        if (get(S.chb4,'Value') == get(S.chb4,'Max'))
            mask1 = S2.regions{ind} .* 0.3; %current region mask
        else
            mask1 = S2.regions{ind} .* 0;
        end
        if (get(S.chb5,'Value') == get(S.chb5,'Max'))
            mask2 = (S2.regions{parentind} - S2.regions{ind}) .* 0.3; %parent mask
        else
            mask2 = (S2.regions{parentind} - S2.regions{ind}) .* 0;
        end
        
        % option to shade all regions that have been marked as stones
        all_reg = uint16(zeros(size(S2.ucm)));
        for kk = 1:length(S2.stones)
            all_reg = all_reg + uint16(S2.regions{S2.stones(kk)});
        end
        
        if (get(S.chb6,'Value') == get(S.chb6,'Max'))
            mask3 = logical(all_reg) .* 0.7;
        else
            mask3 = logical(all_reg) .* 0;
        end
        all_reg(all_reg > 0) = 3; %stone regions = 3
        
        % set colormap, depending on whether stones have been marked yet
        if isempty(S2.stones) == 1
            mp = [1 0 0; 1 1 0; 0 0 1];
        else
            mp = [1 0 0; 1 1 0; 0 0 1; 1 1 1];
        end
        
        ih(1) = imshow(cur_reg,[]);
        set(ih(1),'AlphaData',mask1);
        ih(2) = imshow(par_reg,[]);
        set(ih(2),'AlphaData',mask2);
        if isempty(S2.stones) == 0
            ih(3) = imshow(all_reg,[]);
            set(ih(3),'Alphadata',mask3);
        end
        colormap(mp);
                
        % draw outlines if applicable boxes are checked
        if (get(S.chb2,'Value') == get(S.chb2,'Max'))
            boundparent(S);
        end
        if (get(S.chb3,'Value') == get(S.chb3,'Max'))
            boundsibs(S);
        end
        if (get(S.chb1,'Value') == get(S.chb3,'Max'))
            boundchildren(S);
        end
        hold off

        % zoom to coordinates specified in S2 structure
        xminp = S2.xmin;
        xmaxp = S2.xmax;
        yminp = S2.ymin;
        ymaxp = S2.ymax;
        set(S.ah1,'Xlim',[xminp xmaxp],'Ylim',[yminp ymaxp]);
        
        %call showwholeimage to include bounding box
        showwholeimage(S,xminp,xmaxp,yminp,ymaxp,mp,ind);
        
    end

    function boundsibs(S)
        %draw a boundary around the region's siblings
        S2 = getappdata(S.fh,'mydata');
        ind = S2.current;
        [sibs nSibs] = getSibs(S2,ind);
        for k=1:nSibs
            sibreg = uint16(S2.regions{sibs(k)});
            B = bwboundaries(sibreg);
            for kk=1:length(B)
                bound = B{kk};
                plot(bound(:,2),bound(:,1),'w');
                hold on
            end
        end
    end

    function boundchildren(S)
        %draw a boundary around the region's children
        S2 = getappdata(S.fh,'mydata');
        ind = S2.current;
        [kids nKids] = getChildren(S2,ind);
        for k=1:nKids
            kidreg = uint16(S2.regions{kids(k)});
            B = bwboundaries(kidreg);
            for kk=1:length(B)
                bound = B{kk};
                plot(bound(:,2),bound(:,1),'g');
                hold on
            end
        end
    end

    function boundparent(S)
        %draw a boundary around the region's parent
        S2 = getappdata(S.fh,'mydata');
        ind = S2.current;
        par = getParent(S2,ind);
        % note (DB): i added the next two lines, i think it was a bug
        if (~par)
            par = S2.root;
        end
        parreg = uint16(S2.regions{par});
        B = bwboundaries(parreg);
        for kk=1:length(B)
            bound = B{kk};
            plot(bound(:,2),bound(:,1),'r','Linewidth',3);
            hold on
        end
    end

    function [] = updatevisible(S)
        % update the images, etc, whenever something changes
        S2 = getappdata(S.fh,'mydata');
        set(S.eth1,'String',num2str(S2.current));
        set(S.sth2,'String',strcat('Level: ',num2str(S2.scale(S2.current))));
        set(S.sth3,'String',strcat('Number of siblings: ',num2str(length(S2.siblings{S2.current}))));
        S2 = getappdata(S.fh,'mydata');
        if S2.labels(S2.current) == 0
            set(S.sth4,'String','Label: unassigned');
        elseif S2.labels(S2.current) == 1
            set(S.sth4,'String','Label: stone');
        elseif S2.labels(S2.current) == -1
            set(S.sth4,'String','Label: notstone');
        end
        set(S.sth7,'String',strcat(num2str(length(S2.stones)),' positive regions'));
        set(S.sth8,'String',strcat(num2str(length(S2.notstones)),' negative regions'));
        
        % set the figure background color to be the same as the defaul uicontrol
        % background color
        defaultBackground = get(0,'defaultUicontrolBackgroundColor');
        set(S.fh,'Color',defaultBackground)
        
        %refresh images
        showzoomedimage(S);
    end

end