function [bestInd, bestScore] = bestInPath(S, myPath, mode, descriptors, fields, weights)
% travels up the path indicated by myPath (the result of extractPath)
% returns the index of the best score or zero if no score found
% in case of non unique scores the index returned depends on 'mode'
% if mode is 0 (default) the first value is returned (smallest stone)
% if mode is 1 the last value is returned (largest stone)
% if mode is 2 descriptors properties are used (solidity and circularity)
% fields indicates which records to use in the descriptors.
% default fields are solidity and circularity (usually 5 and 6):
sol = 5; cir = 6; 
% change to 3 and 4 when no area and perimeter are used
% weights contains what percentage to attribute to each field (%).
% the default weights are 1/n, where n is the number of fields.
% author: db

    % check args and set defaults
    if (nargin < 2 || nargin > 6)
        error('usage: [bestInd, bestScore] = bestInPath(S, myPath, [mode, descriptors, fields, weights])');
    end
    if (nargin < 6)
        weights = [];
    end
    if (nargin < 5)
        fields = [];
    end
    if (nargin < 4)
        descriptors = [];
    end
    if (nargin < 3)
        mode = 1;
    end
    
    % for the case where we will use the descriptors
    if (mode == 2)
        % error if they have not been passed
        if (numel(descriptors) == 0)
            error('bestInPath: current mode requires descriptors to be passed!');
        elseif (numel(myPath) ~= size(descriptors, 2))
            error('bestInPath: incorrect descriptors for path!');
        end
        
        % set the default fields (hard-coded above to be solidity and circularity)
        if (numel(fields) == 0)
            fields = false(size(descriptors, 1), 1);
            fields(sol) = true;
            fields(cir) = true;
            numfields = 2;
            
        % error if they were incorrectly passed
        elseif (numel(fields) ~= size(descriptors, 1))
            error('bestInPath: length of fields parameter does not match descriptor!');
        
        % count them
        else
            numfields = sum(fields);
        end
        
        % set the default fields (hard-coded above to be solidity and circularity)
        if (numel(weights) == 0)
            weights = zeros(size(descriptors, 1), 1);
            weights(sol) = 0.5;
            weights(cir) = 0.5;

        % error if they were incorrectly passed
        elseif (numel(weights) ~= size(descriptors, 1))
            error('bestInPath: length of weights parameter does not match descriptor!');

        % error if they don't add up to 1
        elseif (sum(weights) ~= 1)
            error('bestInPath: weights should add up to 1!');
            
        % set them    
        else
            weights(fields) = 1/numfields;
        end
    end
    
    bestScore = 0;
    bestInd = 0;
        
    % index the nodes labeled as stones (return if there are none)
    myCandidates = (S.labels(myPath) == 1);
    numCandidates = sum(myCandidates);
    
    % nothing to do
    if (~numCandidates)
        return;
    
    else
        % check if the scores are unique in which case return the max
        myScores = S.value(myPath(myCandidates));
        numScores = numel(unique(myScores));
        if (numScores == numCandidates)
            [bestScore, bestInd] = max(S.value(myPath));
            bestInd = myPath(bestInd);
            
        % if scores are not unique decide which to return (what to do?)
        else
            switch (mode)
                
                case 0 % this will return the first encountered (smallest stone)
                    [bestScore, bestInd] = max(S.value(myPath));
                    bestInd = myPath(bestInd);            
                
                case 1 % this will return the last encountered (largest stone)
                    flipmyPath = flipud(myPath);
                    [bestScore, bestInd] = max(S.value(flipmyPath));
                    bestInd = flipmyPath(bestInd);
                    
                case 2 % this will use descriptor properties to disambiguate
                    % sort values in descending order
                    [bestScore, bestInd] = sort(S.value(myPath), 'descend');
                    
                    % while the two first values are the same
                    while (bestScore(1) == bestScore(2))
                        
                        % get their descriptors (replace NaN's with 0)
                        desc1 = descriptors(:, bestInd(1));
                        desc2 = descriptors(:, bestInd(2));
                        desc1(isnan(desc1)) = 0;
                        desc2(isnan(desc2)) = 0;
                        
                        % compute score based on solidity and circularity
                        score1 = sum(fields .* weights .* desc1);
                        score2 = sum(fields .* weights .* desc1);
                        
                        %if the first is better
                        if (score1 > score2)    
                            %eliminate the second
                            bestScore = [bestScore(1); bestScore(3:end)];
                            bestInd = [bestInd(1); bestInd(3:end)];
                        
                        %if the second is better
                        else                    
                            %eliminate the first
                            bestScore = bestScore(2:end);
                            bestInd = bestInd(2:end);
                        end
                        
                    end
                    
                    % assign index and score on what remains first in the list
                    bestInd = myPath(bestInd(1));
                    bestScore = bestScore(1);    
                
                otherwise
                    error('bestInPath: no such selection method!');
        end
    end
            
end
