function S = initS(baseName)
% author: pn - db
% initializes main data structure:
%   S.ucm: uint8 matrix containing UCM map
%   S.thresh: threshold used for region extraction (1 = finest)
%   S.regions: cell containing all the regions
%   S.parent: uint16 array containing parent of each region
%   S.scale: uint8 array containing level of formation of each region
%   S.ucm2: uint8 matrix containing UCM2 map
%   S.gPb_thin: uint8 matrix containing thinned gPb map
%   S.names: structure containing relevan file names
%       S.names.baseName: base name for all the files
%       S.names.imageName: image file name
%       S.names.ucmName: ucm map file name
%       S.names.ucm2Name: ucm2 map file name
%       S.names.regName: regions file name
%       S.names.gpbName: gpb map file name
%   S.root: root index of region tree
%   S.leaves: uint16 array containing leaves of region tree
%   S.leafRegNum: uint16 matrix containing region labels for leaves
%   S.children: cell array containing parent of each region
%   S.siblings: cell array containing siblings of each region
%   S.numPaths: number of paths from leaves to root
%   S.paths: structure containing leaf to root path information
%       S.paths.numNodes: uint16 array with number of nodes in paths
%       S.paths.totDist: uint16 array with total distance of paths
%       S.paths.minDist: uint16 array with minimum distance of paths
%       S.paths.maxDist: uint16 array with maximum distance of paths
%       S.paths.avgDist: uint16 array with average distance of paths
%       S.paths.pathValue: double array containing weighted value of paths 
%       S.paths.sortedPaths: uint16 array with sorted indices of paths 
%   S.currentleaf: index of current path
%   S.current: index of current region
%   S.labels: uint16 array of region labels (-1: not stone, 0: not labeled, 1: stone)
%   S.accuracy: double array of region accuracy (returned by SVM classification)
%   S.value: double array of region value (returned by SVM classification)
%   S.stone: uint16 array of region id's for stones
%   S.notstone: uint16 array of region id's for not-stones

    % check args and set defaults
    if (nargin < 1)
        error('initS: insufficient parameters!\n');
    end
    
    fprintf('Assembling tree structure ...\n');
    tStart = tic;
    
    % load regions
    fprintf('Loading regions ...\n');
    tic
    S = load([baseName '_regions.mat']); % need to load as a structure b/c there are nested functions here
    toc

    % initialize names structure
    S.names = struct('baseName', baseName, ...
        'imageName', [baseName '.bmp'], ...
        'ucmName', [baseName '_ucm.bmp'], ...
        'ucm2Name', [baseName '_ucm2.bmp'], ...
        'regName', [baseName '_regions.mat'], ...
        'gpbName', [baseName '_gpb.mat']);
    
    % create the root index
    S.root = uint16(find(S.scale == max(S.scale)));
    
    % create label vectors (root is labeled not stone)
    S.labels = zeros(1,length(S.parent), 'int8');
    S.stones = uint16([]); %row vector of region ids for stones
    S.notstones = uint16(S.root); %row vector of region ids for notstones
    S = setValue(S, S.root, -1);
    
    % create leaves
	S = createLeaves(S);

    % create the children and siblings lists
    S = createChildsSibs(S);

    % create and measure paths
    S = createPaths(S);
    
    % sort paths (optional weights, can be passed)
    S = sortPaths(S);
        
    % start with first sorted lowest level leaf
    S.currentleaf = S.paths.sortedPaths(1);
    S.current = S.leaves(S.currentleaf);

    % done
    toc(tStart)
    fprintf('Done assembling tree\n');
end


